@@ -1,8 +1,5 @@
 bootstrap.pml = function (x, bs = 100, trees = TRUE, multicore=FALSE, mc.cores = NULL, ...) 
 {
-#    multicore=FALSE,
-#    multicore <- mc.cores > 1L
-    
     if(multicore && is.null(mc.cores)){
         mc.cores <- detectCores()
     }
@@ -29,15 +26,8 @@ bootstrap.pml = function (x, bs = 100, trees = TRUE, multicore=FALSE, mc.cores =
     }
     eval.success <- FALSE
     if (!eval.success & multicore) {
-        #  !require(parallel) ||      
-#        if (.Platform$GUI!="X11") {
-#            warning("package 'parallel' not found or GUI is used, 
-#                    bootstrapping is performed in serial")
-#        } 
-#        else {       
-            res <- mclapply(BS, pmlPar, x, trees = trees, ..., mc.cores = mc.cores)
-            eval.success <- TRUE
-#        } 
+        res <- mclapply(BS, pmlPar, x, trees = trees, ..., mc.cores = mc.cores)
+        eval.success <- TRUE
     }
     if (!eval.success) res <- lapply(BS, pmlPar, x, trees = trees, ...)
     if (trees) {
@@ -89,6 +79,8 @@ matchEdges = function(tree1, tree2){
 
 checkLabels <- function(tree, tip){
     ind <- match(tip, tree$tip.label)
+    if (any(is.na(ind)) | length(tree$tip.label) != length(tip))
+        stop("tree has different labels")
     tree$tip.label <- tree$tip.label[ind]
     ind2 <- match(1:length(ind), tree$edge[, 2])
     tree$edge[ind2, 2] <- order(ind)
@@ -97,7 +89,7 @@ checkLabels <- function(tree, tip){
 
 
 plotBS <- function (tree, BStrees, type = "unrooted", bs.col = "black", 
-                    bs.adj = NULL, p=80, ...) 
+                    bs.adj = NULL, p=50, frame="none",...) 
 {
     type <- match.arg(type, c("phylogram", "cladogram", "fan", 
                               "unrooted", "radial"))
@@ -108,13 +100,22 @@ plotBS <- function (tree, BStrees, type = "unrooted", bs.col = "black",
         plot(tree2, type = type, ...)
     }
     else plot(tree, type = type, ...)
-    BStrees <- .uncompressTipLabel(BStrees)
-    if(any(unlist(lapply(BStrees, is.rooted)))){
-        BStrees <- lapply(BStrees, unroot)   
+    
+    if(hasArg(BStrees)){
+        BStrees <- .uncompressTipLabel(BStrees)
+        if(any(unlist(lapply(BStrees, is.rooted)))){
+            BStrees <- lapply(BStrees, unroot)   
+        }
+        x = prop.clades(tree, BStrees)
+        x = round((x/length(BStrees)) * 100)
+        tree$node.label = x
     }
-    x = prop.clades(tree, BStrees)
-    x = round((x/length(BStrees)) * 100)
-    tree$node.label = x
+    else{
+        if(is.null(tree$node.label))stop("You need to supply BStrees or tree needs 
+        needs BS-values as node.label")
+        x <- tree$node.label
+    }
+    
     label = c(rep(0, length(tree$tip)), x)
     ind <- get("last_plot.phylo", envir = .PlotPhyloEnv)$edge[, 
                                                               2]
@@ -128,14 +129,14 @@ plotBS <- function (tree, BStrees, type = "unrooted", bs.col = "black",
         #        browser()
         if (is.null(bs.adj)) 
             bs.adj = c(1, 1)
-        if(length(ind)>0)nodelabels(text = label[ind], node = ind, frame = "none", 
+        if(length(ind)>0)nodelabels(text = label[ind], node = ind, frame = frame, 
                                     col = bs.col, adj = bs.adj, ...)
     }
     else {
         if (is.null(bs.adj)) 
             bs.adj = c(0.5, 0.5)
         ind2 = which(label[ind]>p)
-        if(length(ind2>0))edgelabels(label[ind][ind2],ind2, frame = "none", col = bs.col, 
+        if(length(ind2>0))edgelabels(label[ind][ind2],ind2, frame = frame, col = bs.col, 
                                      adj = bs.adj, ...)
     }
     invisible(tree)
@@ -210,3 +211,80 @@ plotBS.Old <- function (tree, BStrees, type = "unrooted", bs.col = "black",
     }
     invisible(tree)
 }
+
+
+maxCladeCred <- function(x, tree=TRUE, part=NULL, rooted=TRUE){
+    if(inherits(x, "phylo")) x <- c(x)
+    if(!rooted){
+        x <- lapply(x, unroot) 
+        class(x) <- "multiPhylo"
+        x <- .compressTipLabel(x)
+    }    
+    if(is.null(part))pp <- prop.part(x)
+    else pp <- part
+    pplabel <- attr(pp, "labels")
+    if(!rooted)pp <- oneWise(pp)
+    x <- .uncompressTipLabel(x)
+    class(x) <- NULL
+    m <- max(attr(pp, "number"))
+    nb <- log( attr(pp, "number") / m )
+    l <-  length(x)
+    res <- numeric(l)
+    for(i in 1:l){
+        tmp <- checkLabels(x[[i]], pplabel)
+        ppi <- prop.part(tmp)  # trees[[i]]
+        if(!rooted)ppi <- oneWise(ppi)
+        indi <- fmatch(ppi, pp)
+        if(any(is.na(indi))) res[i] <- -Inf
+        else res[i] <- sum(nb[indi])
+    }
+    if(tree) {
+        k <- which.max(res)
+        tr <- x[[k]]
+        attr(tr, "clade.credibility") <- res[k]
+        return(tr)
+    }    
+    res
+}
+
+
+mcc <- maxCladeCred
+
+
+cladeMatrix <- function(x, rooted=FALSE){
+    if(!rooted){
+        x <- .uncompressTipLabel(x)
+        x <- lapply(x, unroot) 
+        class(x) <- "multiPhylo"
+        x <- .compressTipLabel(x)
+    }    
+    pp <- prop.part(x)
+    pplabel <- attr(pp, "labels")
+    if(!rooted)pp <- oneWise(pp)
+    x <- .uncompressTipLabel(x)
+    class(x) <- NULL
+    nnodes <- sapply(x, Nnode)
+    l <-  length(x)
+    from <- cumsum(c(1, nnodes[-l]))
+    to <- cumsum(nnodes)
+    
+    ivec <- integer(to[l])
+    pvec <- c(0,to)
+    
+    res <- vector("list", l)
+    k=1
+    for(i in 1:l){
+        ppi <- prop.part(x[[i]])  
+        if(!rooted)ppi <- oneWise(ppi)
+        indi <- sort(fmatch(ppi, pp))
+        ivec[from[i]:to[i]] = indi
+    }
+    X <- sparseMatrix(i=ivec, p=pvec, dims=c(length(pp),l))
+    list(X=X, prop.part=pp)
+}
+
+
+moving_average <- function(x, window=50){
+     cx <- c(0, cumsum(x))
+     (cx[(window+1):length(cx)] - cx[1:(length(cx)-window)])/(window)
+}
